/************************************************************************************************

 Author        : ADI - Apps            www.analog.com/AduC7060

 Date          : November 2007

 File          : Timer0_IRQ.c

 Hardware      : ADuC706x

 Description   : This program shows how to switch the PLL source oscillator to an external crystal
 				1. On an external IRQ0 interrupt, a routine to switch oscillators is called.
				2. In this routine, the wake-up timer is first enabled with a short time out period (100uS)
				3. The PLLCON register is changed to select the external crystal
				4. Immediately the part is placed in NAP mode.
				5. The wake-up timer will interrupt the core and bring the part out of Nap mode.
		The part is placed in NAP mode to ensure smooth switching of the PLL clock source.
*************************************************************************************************/
// Bit Definitions
#define BIT0  0x01
#define BIT1  0x02
#define BIT2  0x04
#define BIT3  0x08
#define BIT4  0x10
#define BIT5  0x20
#define BIT6  0x40
#define BIT7  0x80
#define BIT8  0x100
#define BIT9  0x200
#define BIT10 0x400
#define BIT11 0x800
#define BIT12 0x1000
#define BIT13 0x2000
#define BIT14 0x4000
#define BIT15 0x8000
#define BIT16 0x10000
#define BIT21 0x200000
#define BIT22 0x400000
#define BIT24 0x1000000
#define BIT26 0x4000000
#define BIT30 0x40000000

void SwitchClockSource( void);
unsigned char ucSwitchClkFlag = 0;
#include<aduc7060.h>

int main(void)
{
	POWKEY1 = 0x1;
	POWCON0 = 0x78;		   // Set core to max CPU speed of 10.24Mhz
	POWKEY2 = 0xF4;

	GP1DAT  = BIT30;		// Configure P1.6 as an output 
	GP0SET = BIT16; 
	ucSwitchClkFlag = 0;
	
	IRQCONE = BIT1;			// Rising edge
	IRQEN =  BIT13;			// Enable Timer 1 IRQ

	
	while(1)
	{
		if (ucSwitchClkFlag == 1)
		{
			SwitchClockSource();		    // Call routine to switch to external crystal
			ucSwitchClkFlag = 0;
		}
		GP1DAT ^= BIT22;	// Toggle LED on Evaluation board (P1.6)
	}
}
void SwitchClockSource( void)
{
	// Initilize Timer 1 (wake-up timer) in Periodic mode with timeout period of 100uS
	T1LD  = 0x80;			// 32768 clock ticks
	T1CON = BIT6 + BIT7;	// Periodic mode, enable timer, 32768hz clock/1
	IRQEN |= BIT4;			// Enable Timer 1 interrupt source

	PLLKEY1 = 0xAA;			// Switch to external crystal
	PLLCON = BIT1;
	PLLKEY2 = 0x55;

	POWKEY1 = 0x1;			// Enter NAP mode and reduce core clock to minimum value
	POWCON0 = 0x73;
	POWKEY2 = 0xF4;

}
void IRQ_Handler(void) __irq  
{
	unsigned int IRQSTATUS = 0;

	IRQSTATUS = IRQSTA;	   				// Read off IRQSTA register

	if ((IRQSTATUS & BIT4) == BIT4)		//Timer 1 interrupt source
	{
		POWKEY1 = 0x1;
		POWCON0 = 0x78;		   			// Set core to max CPU speed of 10.24Mhz
		POWKEY2 = 0xF4;
		IRQCLR |= BIT4;
	}
	if ((IRQSTATUS & BIT13) == BIT13)	//External Interrupt0 source
	{
		 IRQCLRE = BIT13;			    // Clear External Interrupt 0
		 ucSwitchClkFlag = 1;	     
	}
}


